# ============================================================
# Minimum Pool Size Calculator for Citizens' Assemblies
# Version 3.0
# ============================================================

# The calculator helps determine how many volunteers are needed for a 
# demographically representative citizens' assembly. It uses empirically
# validated formulas based on successful real-world assemblies.

# First, we check if the required package is installed and load it
if (!require("openxlsx")) {
  stop("The 'openxlsx' package is not installed. Please install it by running: install.packages('openxlsx')")
}

calculate_minimum_pool_v3 <- function() {
  # Initial message to show the calculator is running
  cat("\nStarting minimum pool size analysis v3.0...\n")
  
  # ============================================================
  # Step 1: Environment Validation
  # ============================================================
  # Before we can process any data, we need to make sure our working
  # environment is properly set up with all required files in place.
  
  if (!dir.exists("data_and_settings")) {
    stop("The 'data_and_settings' folder is missing. Please create it in the same directory as this script.")
  }
  
  if (!file.exists("data_and_settings/categories.xlsx")) {
    stop("categories.xlsx file not found in data_and_settings folder. Please ensure the file exists and has the correct name.")
  }
  
  # ============================================================
  # Step 2: Load and Validate Categories Data
  # ============================================================
  # We load the demographic categories and check that they're properly
  # formatted. This prevents issues that could arise from malformed data.
  
  cat("Loading categories file...\n")
  categories <- tryCatch({
    categories <- read.xlsx("data_and_settings/categories.xlsx", sheet = 1)
    cat("Successfully loaded categories file.\n")
    categories
  }, error = function(e) {
    if (grepl("Sheet 1 not found", e$message)) {
      stop("Could not find Sheet 1 in categories.xlsx. Please ensure your data is in the first sheet.")
    } else {
      stop("Error reading categories file: ", e$message)
    }
  })
  
  # Clean data by removing any extra spaces that might cause issues
  categories <- data.frame(lapply(categories, trimws))
  
  # Check for required columns
  cat("Validating data structure...\n")
  required_columns <- c("category", "feature", "value")
  missing_columns <- setdiff(required_columns, names(categories))
  if (length(missing_columns) > 0) {
    stop(sprintf("Missing required columns in categories file: %s\nRequired columns are: category, feature, value", 
                 paste(missing_columns, collapse = ", ")))
  }
  
  # Validate category names
  if (any(categories$category == "")) {
    stop("Found empty category names. Please ensure all categories have names.")
  }
  if (any(categories$feature == "")) {
    stop("Found empty feature names. Please ensure all features have names.")
  }
  
  # ============================================================
  # Step 3: Validate Demographic Values
  # ============================================================
  # We check that all demographic values make sense and are consistent
  
  # Convert values to numeric and check for invalid entries
  categories$value <- as.numeric(categories$value)
  if (any(is.na(categories$value))) {
    invalid_rows <- which(is.na(categories$value))
    stop(sprintf("Non-numeric values found in the 'value' column at rows: %s", 
                 paste(invalid_rows, collapse = ", ")))
  }
  
  # Check for negative values which aren't logically possible
  if (any(categories$value < 0)) {
    negative_rows <- which(categories$value < 0)
    stop(sprintf("Negative values found in the 'value' column at rows: %s", 
                 paste(negative_rows, collapse = ", ")))
  }
  
  # Check for zero values and provide a warning if found
  zero_rows <- which(categories$value == 0)
  if (length(zero_rows) > 0) {
    cat("\nWarning: Found demographic categories with zero seats allocated:")
    for (row in zero_rows) {
      cat(sprintf("\n  - Category '%s', feature '%s'", 
                  categories$category[row], 
                  categories$feature[row]))
    }
    cat("\nPlease verify if this is intentional.")
    cat("\nZero values mean these demographic groups will have no representation in the assembly.")
    cat("\nConsider whether this aligns with your assembly's goals for inclusivity.\n\n")
  }
  
  # ============================================================
  # Step 4: Calculate and Validate Assembly Size
  # ============================================================
  # We ensure all categories sum to the same total (assembly size)
  
  category_totals <- tapply(categories$value, categories$category, sum)
  
  if (length(unique(category_totals)) > 1) {
    # If totals don't match, provide detailed information about the discrepancy
    totals_table <- data.frame(
      Category = names(category_totals),
      Total = category_totals
    )
    stop(sprintf("Inconsistent totals across categories:\n%s\nAll categories must sum to the same total.",
                 paste(sprintf("%s: %d", totals_table$Category, totals_table$Total), collapse = "\n")))
  }
  
  assembly_size <- category_totals[1]
  cat("\nAssembly size:", assembly_size, "members\n")
  
  if (assembly_size <= 0) {
    stop("Invalid assembly size calculated: ", assembly_size, 
         "\nThe sum of values in each category must be greater than 0")
  }
  
  # ============================================================
  # Step 5: Calculate Pool Size Requirements
  # ============================================================
  # We use empirically validated formulas to calculate the recommended
  # minimum pool size based on assembly size and demographic complexity
  
  # Calculate complexity factor based on number of categories and subcategories
  n_categories <- length(unique(categories$category))
  total_subcategories <- nrow(categories)
  complexity_factor <- (n_categories * total_subcategories) / 161
  
  # Calculate minimum pool size using base ratio and complexity adjustment
  base_minimum <- ceiling(assembly_size * 2.7)  # Base ratio from successful assemblies
  safety_margin <- ceiling(base_minimum * 0.25 * complexity_factor)
  recommended_pool <- base_minimum + safety_margin
  
  # ============================================================
  # Step 6: Present Results
  # ============================================================
  
  cat("\n============================================================")
  cat("\nMINIMUM POOL SIZE ANALYSIS")
  cat("\n============================================================")
  cat("\n")
  cat("\nAssembly composition:")
  cat("\n- Total seats:", assembly_size)
  cat("\n- Number of demographic categories:", n_categories)
  cat("\n- Total demographic subcategories:", total_subcategories)
  cat("\n")
  cat("\nComplexity analysis:")
  cat(sprintf("\n- Complexity factor: %.2f", complexity_factor))
  cat("\n- Base categories:", n_categories)
  cat("\n- Subcategories per category:")
  
  # Show detailed breakdown of subcategories
  for (cat in unique(categories$category)) {
    n_sub <- sum(categories$category == cat)
    cat(sprintf("\n  * %s: %d subcategories", cat, n_sub))
  }
  
  cat("\n")
  cat("\n============================================================")
  cat(sprintf("\nRecommended minimum pool of volunteers: %d", recommended_pool))
  cat("\n============================================================")
  cat("\n")
  
  return(recommended_pool)
}

# Run calculation with error handling
tryCatch({
  result <- calculate_minimum_pool_v3()
}, error = function(e) {
  cat("\nError occurred:", e$message, "\n")
})